@@ -2,7 +2,8 @@ import datetime
 import json
 import celery.loaders.base
 import celery.schedules
-from pyramid.compat import configparser
+import configparser
+from celery import VERSION as celery_version
 from pyramid.exceptions import ConfigurationError
 
 from functools import partial
@@ -21,6 +22,10 @@ SCHEDULE_TYPE_MAP = {
     'timedelta': timedelta,
     'integer': int
 }
+BROKER_TRANSPORT_OPTIONS_MAP = {
+    'visibility_timeout': int,
+    'max_retries': int,
+}
 
 
 def safe_json(get, section, key):
@@ -60,6 +65,9 @@ def get_beat_config(parser, section):
         'schedule': schedule,
     }
 
+    if has_option('options'):
+        config['options'] = safe_json(get, section, 'options')
+
     if has_option('args'):
         config['args'] = safe_json(get, section, 'args')
 
@@ -98,16 +106,25 @@ class INILoader(celery.loaders.base.BaseLoader):
         config_dict = {}
 
         for key, value in self.parser.items('celery'):
-            config_dict[key.upper()] = value
+            config_dict[key] = value
 
-        list_settings = ['CELERY_IMPORTS', 'CELERY_ACCEPT_CONTENT']
+        if celery_version.major > 6:
+            # TODO: Check for invalid settings
+            # that won't be accepted by celery.
+            pass
+
+        list_settings = [
+            'imports',
+            'include',
+            'accept_content',
+        ]
 
         for setting in list_settings:
             if setting in config_dict:
                 split_setting = config_dict[setting].split()
                 config_dict[setting] = split_setting
 
-        tuple_list_settings = ['ADMINS']
+        tuple_list_settings = ['admins']
 
         for setting in tuple_list_settings:
             if setting in config_dict:
@@ -117,19 +134,36 @@ class INILoader(celery.loaders.base.BaseLoader):
 
         beat_config = {}
         route_config = {}
+        dict_settings = [
+            'broker_transport_options',
+        ]
 
         for section in self.parser.sections():
+            sections = section.split(':', 1)
+            # [celery] -> name=celery
+            # [celery:options] -> name=options
+            # [celery:options:task] -> name=options:task
+
+            name = sections[0]
+            if len(sections) > 1:
+                name = sections[1]
+
             if section.startswith('celerybeat:'):
-                name = section.split(':', 1)[1]
                 beat_config[name] = get_beat_config(self.parser, section)
             elif section.startswith('celeryroute:'):
-                name = section.split(':', 1)[1]
                 route_config[name] = get_route_config(self.parser, section)
+            elif name in dict_settings:
+                options = {}
+                for key, value in self.parser.items(section):
+                    if key in BROKER_TRANSPORT_OPTIONS_MAP:
+                        options[key] = BROKER_TRANSPORT_OPTIONS_MAP[key](value)
+
+                config_dict[name] = options
 
         if beat_config:
-            config_dict['CELERYBEAT_SCHEDULE'] = beat_config
+            config_dict['beat_schedule'] = beat_config
 
         if route_config:
-            config_dict['CELERY_ROUTES'] = route_config
+            config_dict['task_routes'] = route_config
 
         return config_dict
